% Estimation using the data compiled by Acemoglu et al. (2012)
% Results are presented in Tables 7 and 8 in Pesaran and Yang (2017) 
clear all;
close all;
fpath = '.\..';     % file path
fname = strcat(fpath, '\data\CC_allyears_new'); 
load(fname);    

pct_set = [0.1,0.2,0.3];   % assuming cut-off values: 10%, 20%, 30%
M = 5;     % for the extrema estimators, compute the M largest deltas
T = 7;     % 1972-2002
CC102 = CC02;
deltaTM = zeros(T,M);     % preallocate space to improve speed
table_N = zeros(T,1);
len_pct = length(pct_set);
beta_GI = zeros(T,len_pct);
se_beta_GI = zeros(T,len_pct);
beta_GI_2ord = zeros(T,len_pct);
se_beta_GI_2ord = zeros(T,len_pct);
beta_mle = zeros(T,len_pct);
se_beta_mle = zeros(T,len_pct);
beta_mle_2ord = zeros(T,len_pct);
se_beta_mle_2ord = zeros(T,len_pct);
alpha_CSN = zeros(T,1);
se_alpha_CSN = zeros(T,1);
xmin_CSN = zeros(T,1);
ntail_CSN = zeros(T,1);
pct_ntail_CSN = zeros(T,1);
alpha_CSN_2ord = zeros(T,1);
se_alpha_CSN_2ord = zeros(T,1);
xmin_CSN_2ord = zeros(T,1);
ntail_CSN_2ord = zeros(T,1);
pct_ntail_CSN_2ord = zeros(T,1);

for t = 1:T   % cross section estimation
    yr = 72 + (t-1)*5;    % data are available every five years
    W = eval(strcat('CC', num2str(yr)));
    W = stdzW(W');
    dN = sum(W,1);
    N = size(W,1);    % number of sectors
    table_N(t) = N;         
    
    %========== Extremum estimator of delta ===========
    results = est_delta(dN,M);
    deltaTM(t,:) = results.deltaM;    
    
    %=========== Power law estimators of beta ==========
    % Based on the first-order interconnections    
    for pi= 1:len_pct    % under the assumed cut-off value
        pct_cut = pct_set(pi);      
        dN_sorted = sort(dN,'descend');
        cut = round(pct_cut*N);
        % GI regresion -- Log-log regression with Gabaix-Ibragimov correction
        b_GI = regress(log((1-0.5:1:cut-0.5)'), [ones(cut,1),log(dN_sorted(1:cut))']);
        [se_b_GI] = (sqrt(2/cut))*b_GI(2);
        beta_GI(t,pi) = -b_GI(2);
        se_beta_GI(t,pi) = -se_b_GI;
        % MLE
        beta_mle(t,pi) = cut/(sum(log(dN_sorted(1:cut))) - cut*log(dN_sorted(cut)));   
        se_beta_mle(t,pi) =  beta_mle(t)/sqrt(cut);          
    end   
  % Infeasible MLE (CSN estimation), calls function plfit.m by Clauset, Shalizi and Newman (2009)
    if t<T
        [alpha_CSN(t), xmin_CSN(t)] = plfit(nonzeros(dN_sorted),'finite');
    elseif t==T   
        [alpha_CSN(t), xmin_CSN(t)] = plfit(nonzeros(dN_sorted),'nosmall');   
                 % This option is chosen following Acemoglu et al. (2012)
    end
    ntail_CSN(t) = length(find(dN_sorted > xmin_CSN(t)));
    se_alpha_CSN(t) = (alpha_CSN(t)-1)/sqrt(ntail_CSN(t));
    beta_CSN = alpha_CSN - 1;
    pct_ntail_CSN(t) = ntail_CSN(t)/N;   % estimated cut-off value
        
    
    % Based on the second-order interconnections
    % GI regresion
    for pi= 1:len_pct
        pct_cut = pct_set(pi);
        dN2 = zeros(size(W,1));
        dN = sum(W,1);  % 1 by N row vector
        for i = 1:N
            dN2(i) = W(:,i)'*dN';
        end
        dN2_sorted = sort(dN2,'descend');
        cut = round(pct_cut*N);
        b_GI_2ord = regress(log((1-0.5:1:cut-0.5)'), [ones(cut,1),log(dN2_sorted(1:cut))']);
        [se_b_GI_2ord] = (sqrt(2/cut))*b_GI_2ord(2);
        beta_GI_2ord(t,pi) = -b_GI_2ord(2);
        se_beta_GI_2ord(t,pi) = -se_b_GI_2ord;
         % MLE
        beta_mle_2ord(t,pi) = cut/(sum(log(dN2_sorted(1:cut))) - cut*log(dN2_sorted(cut)));   
        se_beta_mle_2ord(t,pi) =  beta_mle_2ord(t)/sqrt(cut);   
    end    
    % Infeasible MLE (CSN estimation)
    if t<7
        [alpha_CSN_2ord(t), xmin_CSN_2ord(t)] = plfit(nonzeros(dN2_sorted),'finite');
    elseif t==7
        [alpha_CSN_2ord(t), xmin_CSN_2ord(t)] = plfit(nonzeros(dN2_sorted),'nosmall'); 
    end
    ntail_CSN_2ord(t) = length(find(dN2_sorted>xmin_CSN_2ord(t)));
    se_alpha_CSN_2ord(t) = (alpha_CSN_2ord(t)-1)/sqrt(ntail_CSN_2ord(t));
    beta_CSN_2ord = alpha_CSN_2ord - 1;
    pct_ntail_CSN_2ord(t) = ntail_CSN_2ord(t)/N;
end

% Compare delta_max and inverse of beta
delta_PY = deltaTM(:,1);   % the largest delta
delta_GI = 1./beta_GI;     % inverse of beta
se_delta_GI = se_beta_GI./beta_GI.^2; 
delta_mle = 1./beta_mle;
se_delta_mle = se_beta_mle./beta_mle.^2;
delta_CSN = 1./beta_CSN;
se_delta_CSN = se_alpha_CSN./beta_CSN.^2;

delta_GI_2ord = 1./beta_GI_2ord;
se_delta_GI_2ord = se_beta_GI_2ord./beta_GI_2ord.^2;
delta_mle_2ord = 1./beta_mle_2ord;
se_delta_mle_2ord = se_beta_mle_2ord./beta_mle_2ord.^2;
delta_CSN_2ord = 1./beta_CSN_2ord;
se_delta_CSN_2ord = se_alpha_CSN_2ord./beta_CSN_2ord.^2;


%% Formatting output
table_1ord = zeros(T,2*len_pct+4);    % Table 7 in the paper
for t = 1:T
    table_1ord(2*t-1,:) = [table_N(t),delta_PY(t,:),delta_GI(t,:),delta_mle(t,:),delta_CSN(t,:),pct_ntail_CSN(t,:)];
    table_1ord(2*t,:) = [0,0,se_delta_GI(t,:),se_delta_mle(t,:),se_delta_CSN(t,:),0];
end

table_2ord = zeros(T,2*len_pct+4);    % Table 8 in the paper
for t = 1:T
    table_2ord(2*t-1,:) = [table_N(t),delta_PY(t,:),delta_GI_2ord(t,:),delta_mle_2ord(t,:),delta_CSN_2ord(t,:),pct_ntail_CSN_2ord(t,:)];
    table_2ord(2*t,:) = [0,0,se_delta_GI_2ord(t,:),se_delta_mle_2ord(t,:),se_delta_CSN_2ord(t,:),0];
end

table = [table_1ord;zeros(2,size(table_1ord,2));table_2ord];
% fname = strcat(fpath,'\output\results_ACOTdata.csv');
% csvwrite(fname,table,0,0);
